@@ -4,11 +4,14 @@
 
 #pragma once
 
+#include <QMimeData>
 #include <QObject>
 
 #include <freerdp/freerdp.h>
 #include <freerdp/server/cliprdr.h>
 
+#include <mutex>
+
 #include "krdp_export.h"
 
 namespace KRdp
@@ -34,10 +37,15 @@ public:
 
     bool enabled();
 
+    void setServerData(const QMimeData *data);
+
+    Q_SIGNAL void clientDataChanged();
+    std::unique_ptr<QMimeData> getClipboard() const;
+
 private:
+    void sendServerData();
+
     class Private;
     const std::unique_ptr<Private> d;
-
-    bool m_enabled;
 };
 }
