@@ -28,16 +28,17 @@ using namespace Qt::StringLiterals;
 K_PLUGIN_CLASS_WITH_JSON(KRDPServerConfig, "kcm_krdpserver.json")
 
 static const QString passwordServiceName = QLatin1StringView("KRDP");
-static const QString dbusSystemdDestination = u"org.freedesktop.systemd1"_qs;
-static const QString dbusSystemdPath = u"/org/freedesktop/systemd1"_qs;
-static const QString dbusKrdpServerServicePath = u"/org/freedesktop/systemd1/unit/plasma_2dkrdp_5fserver_2eservice"_qs;
-static const QString dbusSystemdUnitInterface = u"org.freedesktop.systemd1.Unit"_qs;
-static const QString dbusSystemdManagerInterface = u"org.freedesktop.systemd1.Manager"_qs;
-static const QString dbusSystemdPropertiesInterface = u"org.freedesktop.DBus.Properties"_qs;
+static const QString dbusSystemdDestination = u"org.freedesktop.systemd1"_s;
+static const QString dbusSystemdPath = u"/org/freedesktop/systemd1"_s;
+static const QString dbusKrdpServerServicePath = u"/org/freedesktop/systemd1/unit/app_2dorg_2ekde_2ekrdpserver_2eservice"_s;
+static const QString dbusSystemdUnitInterface = u"org.freedesktop.systemd1.Unit"_s;
+static const QString dbusSystemdManagerInterface = u"org.freedesktop.systemd1.Manager"_s;
+static const QString dbusSystemdPropertiesInterface = u"org.freedesktop.DBus.Properties"_s;
 
 KRDPServerConfig::KRDPServerConfig(QObject *parent, const KPluginMetaData &data)
     : KQuickManagedConfigModule(parent, data)
     , m_serverSettings(new KRDPServerSettings(this))
+    , m_usersModel(new UsersModel(m_serverSettings, this))
 {
     setButtons(Help | Apply | Default);
 
@@ -51,7 +52,7 @@ KRDPServerConfig::KRDPServerConfig(QObject *parent, const KPluginMetaData &data)
     QDBusConnection::sessionBus().connect(dbusSystemdDestination,
                                           dbusKrdpServerServicePath,
                                           dbusSystemdPropertiesInterface,
-                                          u"PropertiesChanged"_qs,
+                                          u"PropertiesChanged"_s,
                                           this,
                                           SLOT(servicePropertiesChanged()));
 }
@@ -107,10 +108,10 @@ void KRDPServerConfig::deletePasswordFromWallet(const QString &user)
 void KRDPServerConfig::addUser(const QString &username, const QString &password)
 {
     if (!username.isEmpty()) {
-        auto userList = m_serverSettings->users();
+        auto userList = m_usersModel->users();
         userList.append(username);
         writePasswordToWallet(username, password);
-        m_serverSettings->setUsers(userList);
+        m_usersModel->setUsers(userList);
     }
     save();
 }
@@ -123,14 +124,14 @@ void KRDPServerConfig::modifyUser(const QString &oldUsername, const QString &new
     }
     // If we have new username, we're removing the old one and adding the new one
     if (!newUsername.isEmpty()) {
-        auto userList = m_serverSettings->users();
+        auto userList = m_usersModel->users();
         if (userList.contains(oldUsername)) {
             userList.removeAll(oldUsername);
         }
         userList.append(newUsername);
         deletePasswordFromWallet(oldUsername);
         writePasswordToWallet(newUsername, newPassword);
-        m_serverSettings->setUsers(userList);
+        m_usersModel->setUsers(userList);
     }
     // We change the password of the old user
     else {
@@ -145,19 +146,19 @@ void KRDPServerConfig::deleteUser(const QString &username)
 {
     // Remove the old username
     if (!username.isEmpty()) {
-        auto userList = m_serverSettings->users();
+        auto userList = m_usersModel->users();
         if (userList.contains(username)) {
             userList.removeAll(username);
         }
         deletePasswordFromWallet(username);
-        m_serverSettings->setUsers(userList);
+        m_usersModel->setUsers(userList);
     }
     save();
 }
 
 bool KRDPServerConfig::userExists(const QString &username)
 {
-    return m_serverSettings->users().contains(username);
+    return m_usersModel->users().contains(username);
 }
 
 void KRDPServerConfig::save()
@@ -171,9 +172,9 @@ void KRDPServerConfig::defaults()
     // Do not reset list of users. This would not be needed
     // if we can get the list of users from wallet, instead
     // of saving it in this config
-    auto userList = m_serverSettings->users();
+    auto userList = m_usersModel->users();
     KQuickManagedConfigModule::defaults();
-    m_serverSettings->setUsers(userList);
+    m_usersModel->setUsers(userList);
 }
 
 void KRDPServerConfig::createRestoreToken()
@@ -230,6 +231,12 @@ QString KRDPServerConfig::hostName() const
     return info.localHostName();
 }
 
+bool KRDPServerConfig::managementAvailable() const
+{
+    static bool managementAvailable = QDBusConnection::sessionBus().interface()->isServiceRegistered(u"org.freedesktop.systemd1"_s);
+    return managementAvailable;
+}
+
 void KRDPServerConfig::toggleAutoconnect(const bool enabled)
 {
     qDebug(KRDPKCM) << "Setting KRDP Server service autostart on login to " << enabled << "over QDBus";
@@ -237,11 +244,11 @@ void KRDPServerConfig::toggleAutoconnect(const bool enabled)
     auto msg = QDBusMessage::createMethodCall(dbusSystemdDestination,
                                               dbusSystemdPath,
                                               dbusSystemdManagerInterface,
-                                              enabled ? u"EnableUnitFiles"_qs : u"DisableUnitFiles"_qs);
+                                              enabled ? u"EnableUnitFiles"_s : u"DisableUnitFiles"_s);
     if (enabled) {
-        msg.setArguments({QStringList(u"app-org.kde.krdpserver.service"_qs), false, true});
+        msg.setArguments({QStringList(u"app-org.kde.krdpserver.service"_s), false, true});
     } else {
-        msg.setArguments({QStringList(u"app-org.kde.krdpserver.service"_qs), false});
+        msg.setArguments({QStringList(u"app-org.kde.krdpserver.service"_s), false});
     }
     QDBusConnection::sessionBus().asyncCall(msg);
 
@@ -252,9 +259,9 @@ void KRDPServerConfig::toggleAutoconnect(const bool enabled)
 
 void KRDPServerConfig::toggleServer(const bool enabled)
 {
-    auto msg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdUnitInterface, enabled ? u"Start"_qs : u"Stop"_qs);
+    auto msg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdUnitInterface, enabled ? u"Start"_s : u"Stop"_s);
 
-    msg.setArguments({u"replace"_qs});
+    msg.setArguments({u"replace"_s});
     qDebug(KRDPKCM) << "Toggling KRDP Server to " << enabled << "over QDBus";
     QDBusConnection::sessionBus().asyncCall(msg);
 
@@ -266,8 +273,8 @@ void KRDPServerConfig::toggleServer(const bool enabled)
 void KRDPServerConfig::restartServer()
 {
     qDebug(KRDPKCM) << "Restarting KRDP Server";
-    auto restartMsg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdUnitInterface, u"Restart"_qs);
-    restartMsg.setArguments({u"replace"_qs});
+    auto restartMsg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdUnitInterface, u"Restart"_s);
+    restartMsg.setArguments({u"replace"_s});
     auto pendingCall = QDBusConnection::sessionBus().asyncCall(restartMsg);
     auto watcher = new QDBusPendingCallWatcher(pendingCall, this);
     connect(watcher, &QDBusPendingCallWatcher::finished, this, [&](QDBusPendingCallWatcher *w) {
@@ -281,24 +288,24 @@ void KRDPServerConfig::generateCertificate()
     if (!m_serverSettings->certificate().isEmpty() || !m_serverSettings->certificateKey().isEmpty()) {
         return;
     }
-    QDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation)).mkpath(u"krdpserver"_qs);
-    QString certificatePath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + u"/krdpserver/krdp.crt"_qs);
-    QString certificateKeyPath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + u"/krdpserver/krdp.key"_qs);
+    QDir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation)).mkpath(u"krdpserver"_s);
+    QString certificatePath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + u"/krdpserver/krdp.crt"_s);
+    QString certificateKeyPath(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + u"/krdpserver/krdp.key"_s);
     qDebug(KRDPKCM) << "Generating certificate files to: " << certificatePath << " and " << certificateKeyPath;
     QProcess sslProcess;
-    sslProcess.start(u"openssl"_qs,
+    sslProcess.start(u"openssl"_s,
                      {
-                         u"req"_qs,
-                         u"-nodes"_qs,
-                         u"-new"_qs,
-                         u"-x509"_qs,
-                         u"-keyout"_qs,
+                         u"req"_s,
+                         u"-nodes"_s,
+                         u"-new"_s,
+                         u"-x509"_s,
+                         u"-keyout"_s,
                          certificateKeyPath,
-                         u"-out"_qs,
+                         u"-out"_s,
                          certificatePath,
-                         u"-days"_qs,
-                         u"1"_qs,
-                         u"-batch"_qs,
+                         u"-days"_s,
+                         u"1"_s,
+                         u"-batch"_s,
                      });
     sslProcess.waitForFinished();
 
@@ -320,8 +327,8 @@ void KRDPServerConfig::generateCertificate()
 void KRDPServerConfig::checkServerRunning()
 {
     // Checks if there is PID, and if there is, process is running.
-    auto msg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdPropertiesInterface, u"Get"_qs);
-    msg.setArguments({u"org.freedesktop.systemd1.Service"_qs, u"MainPID"_qs});
+    auto msg = QDBusMessage::createMethodCall(dbusSystemdDestination, dbusKrdpServerServicePath, dbusSystemdPropertiesInterface, u"Get"_s);
+    msg.setArguments({u"org.freedesktop.systemd1.Service"_s, u"MainPID"_s});
 
     QDBusPendingCall pcall = QDBusConnection::sessionBus().asyncCall(msg);
     auto watcher = new QDBusPendingCallWatcher(pcall, this);
